#!/bin/sh

#
#

CURL=./curl
curlopts="--anyauth -s -S --connect-timeout 10 --max-time 10"

NIC=
MAC=
IFINDEX=
CONNECTED=
SPEED=
DHCP=
IP=
MASK=
GW=
DNS=

USERNAME=root
PASSWORD=pass

# function to get username and password
get_creds()
{
    # curl needs to know the username/password of the camera so find that out the hard way
    line=`grep password ./vivotekpi.xml`
    
    rc=$?
    if [ $rc -ne 0 ]
    then
        echo "Error: failed to get credentials with error $rc"
        exit $rc
    fi

    #
    # this is kind of convoluted since ash doesn't do all the string matching of bash
    for word in $line
    do
    case $word in
      password*) pass=`echo $word | cut -d \= -f2` ;;
  
      username*) user=`echo $word | cut -d \= -f2` ;;
    esac
    done
    
    ##
    # now strip leading and trailing " from username/password
    #
    user=${user#\"}
    USERNAME=${user%\"}
    pass=${pass#\"}
    PASSWORD=${pass%\"}
}

# function to print get network settings
print_network_config()
{
    USER=$1
    PWD=$2

    URL="http://127.0.0.1/cgi-bin/admin/getparam.cgi?"
    URL="${URL}network_ipaddress&network_subnet&network_router&network_dns1&network_resetip&status_eth_i0"

    output=`$CURL $curlopts -u $USER:$PWD "$URL" 2>&1`

    rc=$?
    if [ $rc -ne 0 ]
    then
        echo "Error: failed to get network settings with error $rc"
        echo "$output"
        exit $rc
    fi

    for l in $output
    do
        case $l in
            network_resetip=*) DHCP=`echo $l | sed -n 's/.*='\''\(.*\)'\''/\1/p' | tr -d "\r"` ;;
            network_ipaddress=*) IP=`echo $l | sed -n 's/.*='\''\(.*\)'\''/\1/p' | tr -d "\r"` ;;
            network_router=*) GW=`echo $l | sed -n 's/.*='\''\(.*\)'\''/\1/p' | tr -d "\r"` ;;
            network_subnet=*) MASK=`echo $l | sed -n 's/.*='\''\(.*\)'\''/\1/p' | tr -d "\r"` ;;
            network_dns1=*) DNS=`echo $l | sed -n 's/.*='\''\(.*\)'\''/\1/p' | tr -d "\r"` ;;
            [0-9]*base*) SPEED=`echo $l | sed -n 's/\([0-9]*\)base.*/\1/p'` ;;  # Mozart
            [0-9]*,) SPEED=`echo $l | sed -n 's/\([0-9]*\),.*/\1/p'` ;;         # Vivaldi
        esac
    done
    
    if [ -z "$IP" ]
    then
        echo "Error: Failed to get settings. $output"
        exit 1
    fi
    
    MAC=`cat /sys/class/net/eth0/address`
    
    IFINDEX=`cat /sys/class/net/eth0/ifindex`
    if [ -z "$IFINDEX" ]; then
        IFINDEX=2
    fi
    
    CONNECTED=`cat /sys/class/net/eth0/carrier`
    
    if [ -z "$SPEED" ]; then
        SPEED=100
    fi
    
    echo "reboot=1"
    echo "nic=eth0"
    echo "mac=$MAC"
    echo "index=$IFINDEX"
    echo "connected=$CONNECTED"
    echo "linkspeed=$SPEED"
    echo "dhcp=$DHCP"
    echo "ip=$IP"
    echo "netmask=$MASK"
    echo "gateway=$GW"
    echo "dns=$DNS"
}
# # ---- end helper functions above ---- # #

if [ $# -lt 1 ]
then
    echo "Error: Must specify get or set parameter"
    exit 1
fi

get_creds
if [ $1 = "get" ]
then
    print_network_config $USERNAME $PASSWORD
elif [ $1 = "set" ]
then
    if [ $# -lt 2 ]
    then
        echo "Error: Must specify network interface name"
        exit 1
    fi
    
    NIC=$2
    shift 2 #now $1 is the first parameter after interface name

    do_update=0
    while :;
    do
        case $1 in
            -ip)
                if [ -n "$2" ]; then
                    IP=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -ip"
                    exit 1
                fi
                ;;
            -mask)
                if [ -n "$2" ]; then
                    MASK=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -mask"
                    exit 1
                fi
                ;;
            -gw)
                if [ -n "$2" ]; then
                    GW=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -gw"
                    exit 1
                fi
                ;;
            -dns)
                if [ -n "$2" ]; then
                    DNS=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -dns"
                    exit 1
                fi
                ;;
            -dhcp)
                if [ -n "$2" ]; then
                    DHCP=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -dhcp"
                    exit 1
                fi
                ;;
            -?*)
                echo "Error: unknown option $1"
                exit 1
                ;;
            *) break
        esac
        shift
    done

    if [ $do_update -eq 0 ]
    then
        echo "Error: no valid update parameters found"
        exit 1
    fi

    #build url for curl
    URL="http://127.0.0.1/cgi-bin/admin/setparam.cgi?"
    if [ -n "$DHCP" ]; then
        URL="${URL}network_resetip=$DHCP&"
    fi
    if [ -n "$IP" ]; then
        URL="${URL}network_ipaddress=$IP&"
    fi
    if [ -n "$MASK" ]; then
        URL="${URL}network_subnet=$MASK&"
    fi
    if [ -n "$GW" ]; then
        URL="${URL}network_router=$GW&"
    fi
    if [ -n "$DNS" ]; then
        URL="${URL}network_dns1=$DNS&"
    fi

    #indicate network restart
    URL="${URL}network_restart=1"
    #if Mozart or switching to DHCP then reboot is needed
    if [ "$DHCP" = "1" -o -e /lib/ld-uClibc.so.0 ]; then
        URL="${URL}&system_reset=60"
    fi
    output=`$CURL $curlopts -u $USERNAME:$PASSWORD "$URL" 2>&1`
    
    if [ $? -ne 0 ]
    then
        echo "Error: curl command to update settings using ($URL) failed with error $?"
        exit $?
    fi
    
    # check that output contains network_, error otherwise
    case $output in
        *network_*) ;;
        *)
            echo "Error: Failed to update network settings via URL ($URL)."
            echo "$output"
            exit 1
            ;;
    esac

    print_network_config $USERNAME $PASSWORD    
fi

